/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.components.DCFrame;
import net.sf.dc.console.components.DCTable;
import net.sf.dc.console.components.Layout;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DCModule;
import net.sf.dc.core.modules.DCModules;
import net.sf.dc.core.objects.DCObject;
import net.sf.dc.freedb.Freedb;
import net.sf.dc.helpsystem.JavaHelp;
import net.sf.dc.processes.http.InternetUpdateProcess;
import net.sf.dc.resources.DCLabels;
import net.sf.dc.resources.DCMessages;
import net.sf.dc.resources.DCTooltips;
import net.sf.dc.wf.WorkFlow;
import net.sf.dc.wf.requests.UpdateItemFormRequest;
import net.sf.dc.wf.requests.UpdateRowRequest;
import net.sf.ext.jflexsett.settings.Settings;
import net.sf.ext.util.Resources;
import net.sf.ext.util.Utilities;

/**
 * Base panel for all internet search forms
 * @author Robert Jan van der Waals
 * @since 1.4
 * @version 1.5
 */
public abstract class InternetSearchForm extends DCFrame {
    
    protected final WorkFlow wf = new WorkFlow();
    private ArrayList items = new ArrayList();
    
    protected JCheckBox checkOverwrite;
    protected JCheckBox checkBroadband;

    private JTextArea textLog = ComponentFactory.getTextArea();
    private JProgressBar progressBar = new JProgressBar();
    private JPanel panelSettings = new JPanel(false);
    
    protected JButton buttonStopSearch;
    protected boolean startSearchOnOpen = true;
    protected DCTable table;
    
    // the background color
    protected final Color color = 
        Settings.getValueAsColor(Repository.settings.stColor);
    
    private final ItemForm itemForm;
    protected final DCObject dco;
    
    // the current module
    protected final DCModule module;

    // the requestor class
    protected final int tab;
    
    protected InternetUpdateProcess process;
    
    protected boolean cancelled = false;
    
    /**
     * Initializes this form for a specific data crow object. 
     * The results will be passed back on to the item form
     * @param dco item, can be null
     * @param itemForm item form, can be null
     */
    public InternetSearchForm(JFrame parent, DCObject dco, ItemForm itemForm) {
        super(parent);
        
        this.dco = dco;
        this.itemForm = itemForm;
        this.tab = -1;
        this.module = 
            DCModules.getModule(Settings.getValueAsInt(Repository.settings.stModule));
        
        initialize();
    }
    
    /**
     * Initializes this form for a specific data crow object. 
     * The results will be passed back on to the specified class.
     * The rownumber indicates which row in table of the class should
     * be updated
     * @param dco 
     * @param tab insert or search tab
     * @param rowNumber UI table row number
     */
    public InternetSearchForm(JFrame parent, DCObject dco, int tab) {
        super(parent);
        
        this.dco = dco;
        this.tab = tab;
        this.itemForm = null;
        this.module = 
            DCModules.getModule(Settings.getValueAsInt(Repository.settings.stModule));
        
        initialize();
    }
    
    public JPanel getSettingsPanel() {
    	return panelSettings;
    }
    
    public void setVisible(boolean b) {
    	cancelled = false;
        table.cancelEdit();
        super.setVisible(b);
    }
    
    /**
     * Initializes the progress bar. Sets the maximum value
     * @param maxValue maximum value
     */
    public void initProgressBar(int maxValue) {
        progressBar.setValue(0);
        progressBar.setMaximum(maxValue);
    }
    
    /**
     * Updates the value of the progress bar
     * @param value progress value
     */
    public void updateProgressBar(int value) {
        progressBar.setValue(value);
    }  
    
    /**
     * Adds an object to the table
     * @param dco
     */
    public void addObject(DCObject result) {
        if (!cancelled) {
            if (dco != null) {
            	result.setValue(DCObject._ID, dco.getID());
            }
        	table.addObject(result);
            items.add(result);
        }
    }
    
    /**
     * Adds a log message
     * @param message message to be added
     */
    public void addMessage(String message) {
        textLog.insert(message + '\n', 0);
    }    
    
    /**
     * Returns the selected data (in the table)
     */
    public DCObject getSelectedData() {
        int row = table.getSelectedRow();

        if (row < 0) {
            new MessageBox(Resources.getText(DCMessages.msgSelectRowForTransfer),
                           MessageBox._WARNING);
            return null;
        }  
        
        DCObject o = (DCObject) items.get(row);
    	o.setValue(DCObject._ID, o.getID());
        return o;
    }   
    
    /**
     * Opens an item form for the selected item
     */
    private void openItem() {
        int selectedRow = table.getSelectedRow();
        if (selectedRow == -1) {
            new MessageBox(Resources.getText(DCMessages.msgSelectRowToOpen),
                           MessageBox._WARNING);
            return;
        }
        
        DCObject o = getSelectedData();
        if (o != null) {
        	new ItemForm(this, true, false, o, false, false, getClass(), -1);
        }
    }  
    
    /**
     * Cancels the internet search
     */
    private void stopSearch() {
        cancelled = true;
        if (process != null) {
        	process.stopSearch();
        }
        
        Freedb.cancel();
        
        addMessage(Resources.getText(DCMessages.msgStoppedSearch));
        setSearchMode();
    }
    
    /**
     * Closes this form and applies all settings
     */
    protected void close() {
        saveSettings();

    	stopSearch();
        dispose();
        setVisible(false);
    }
    
    protected void saveSettings() {
        Settings.setValue(Repository.settings.stODBOverwriteValues,
                new Boolean(checkOverwrite.isSelected()));
        Settings.setValue(Repository.settings.stBroadband,
                new Boolean(checkBroadband.isSelected()));
        Settings.setValue(Repository.settings.stImdbSearchFormViewSize,
                getSize());     
    }
    
    /**
     * Transfers the selected result to the requestor 
     */
    public void update() {
        table.cancelEdit();
        DCObject o = getSelectedData();
        if (o != null) {
            if (itemForm != null) {
                new UpdateItemFormRequest(itemForm, o, checkOverwrite.isSelected()).
                    execute(new ArrayList());
            } else {
                new UpdateRowRequest(tab, o, checkOverwrite.isSelected(), false).
                    execute(new ArrayList());
            }
            close();
        }
    }   
 
    
    public void addNew() {
        table.cancelEdit();
        DCObject o = getSelectedData();
        
        if (o != null) {
            o.setIDs();
            module.getInsertPanel().addObject(o);
            DataCrow.mainFrame.setSelectedTab(MainFrame._INSERTTAB);
            close();
        }
    }
    
    /**
     * Clears the content of the UI table
     */
    private void clearTable() {
        items.clear();
        table.cancelEdit();
    	table.setRowCount(0);
        updateProgressBar(0);
        textLog.setText("");
    }
    
    private void initialize() {
        buildDialog();
        
        JavaHelp.setHelpIndex("dc.Searching.Odb");
        
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                close();
            }
        });        
        
        setIconImage(IconLibrary._icoInternet.getImage());
        
        setDefaultValues();
        
        Dimension location = Utilities.getCenteredWindowLocation(getSize());
        setLocation(location.width, location.height);
        
        setSearchMode();        
        setVisible(true);
        
        if (dco != null && startSearchOnOpen) {
            startSearch();
        }
    }
    
    public abstract void startSearch();
    public abstract void setSearchMode();
    public abstract void setCancelMode();
    protected abstract void setDefaultValues();
    protected abstract JPanel getSearchPanel();    
    
    private void buildDialog() {
        boolean overwrite = Settings.getValueAsBoolean(
                Repository.settings.stODBOverwriteValues);
        boolean broadband = Settings.getValueAsBoolean(
                Repository.settings.stBroadband);        
        
        setResizable(true);
        getContentPane().setLayout(Layout.getGBL());
        getContentPane().setBackground(color);

        //**********************************************************
        //Settings
        //**********************************************************
        JPanel panelSettings = new JPanel(false);
        panelSettings.setLayout(Layout.getGBL());
        panelSettings.setBackground(color);
        
        checkOverwrite = ComponentFactory.getCheckBox(
                         Resources.getText(DCLabels.lblOverwriteExistingValues));
        checkOverwrite.setBackground(color);
        checkOverwrite.setToolTipText(Resources.getText(DCTooltips.tpOverwriteExistingValues));
        checkOverwrite.setSelected(overwrite);
        
        checkBroadband = ComponentFactory.getCheckBox(
                         Resources.getText(DCLabels.lblBroadbandConnection));
        checkBroadband.setBackground(color);
        checkBroadband.setSelected(broadband);
        
        panelSettings.add(checkOverwrite, Layout.getGBC( 0, 0, 10, 1, 1.0, 1.0
                         ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
						  new Insets(5, 5, 5, 5), 0, 0));
        panelSettings.add(checkBroadband, Layout.getGBC( 0, 1, 10, 1, 1.0, 1.0
                         ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
						  new Insets(5, 5, 5, 5), 0, 0));
        
        panelSettings.setBorder(ComponentFactory.getTitleBorder(Resources.getText(DCLabels.lblSettings)));        
        
        //**********************************************************
        //Search panel
        //**********************************************************
        JPanel panelSearch = getSearchPanel();
        
        //**********************************************************
        //Progress panel
        //**********************************************************
        JPanel panelProgress = new JPanel(false);
        panelProgress.setLayout(Layout.getGBL());
        panelProgress.setBackground(color);
        panelProgress.add(progressBar, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                         ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                          new Insets(5, 5, 5, 5), 0, 0)); 
        
        //**********************************************************
        //Log Panel
        //**********************************************************
        JPanel panelLog = new JPanel(false);
        panelLog.setLayout(Layout.getGBL());
        panelLog.setBackground(color);
        
        JScrollPane scroller = new JScrollPane(textLog);
        scroller.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        panelLog.add(scroller, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                    ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                     new Insets(5, 5, 5, 5), 0, 0));
        
        panelLog.setBorder(ComponentFactory.getTitleBorder(
        	               Resources.getText(DCLabels.lblLog)));
        
        //**********************************************************
        //Result panel
        //**********************************************************
        JPanel panelResults = new JPanel(false);
        panelResults.setLayout(Layout.getGBL());
        panelResults.setBackground(color);
        
        table = ComponentFactory.getDCTable(module, true, false);
            
        JScrollPane scrollTable = new JScrollPane(table);
        table.addMouseListener(new TableMouseListener());
        scrollTable.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        
        table.removeHiddenFields(module.getDCObject().getHiddenFields());
        
        panelResults.add(scrollTable,  Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                        ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                         new Insets(5, 5, 5, 5), 0, 0));
         
        //**********************************************************
        //Actions panel
        //**********************************************************        
        JPanel panelActions = new JPanel(false);
        panelActions.setLayout(Layout.getGBL());
        panelActions.setBackground(color);  
        
        JButton buttonUpdate = ComponentFactory.getButton(Resources.getText(DCLabels.lblUpdate));
        JButton buttonAddNew = ComponentFactory.getButton(Resources.getText(DCLabels.lblAddNew));
        buttonStopSearch =  ComponentFactory.getButton(Resources.getText(DCLabels.lblStop));
        JButton buttonClear = ComponentFactory.getButton(Resources.getText(DCLabels.lblClear));
        JButton buttonClose = ComponentFactory.getButton(Resources.getText(DCLabels.lblClose));
        
        buttonClose.setToolTipText(Resources.getText(DCTooltips.tpClose));
        buttonUpdate.setToolTipText(Resources.getText(DCTooltips.tpUpdate));
        buttonAddNew.setToolTipText(Resources.getText(DCTooltips.tpAddNew));
        
        buttonUpdate.addActionListener(new UpdateAction());
        buttonClose.addActionListener(new CancelAction());
        buttonClear.addActionListener(new ClearAction());
        buttonAddNew.addActionListener(new AddNewAction());
        buttonStopSearch.addActionListener(new StopSearchAction());        
        
        if (dco != null) {
        	panelActions.add(buttonUpdate,    Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
        			        ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
							 new Insets(5, 5, 5, 5), 0, 0));
        }
        
        panelActions.add(buttonAddNew,    Layout.getGBC( 1, 0, 1, 1, 1.0, 1.0
                        ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                         new Insets(5, 5, 5, 5), 0, 0));
        panelActions.add(buttonStopSearch,Layout.getGBC( 2, 0, 1, 1, 1.0, 1.0
                        ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
						 new Insets(5, 5, 5, 5), 0, 0));
        panelActions.add(buttonClear,     Layout.getGBC( 3, 0, 1, 1, 1.0, 1.0
                        ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
						new Insets(5, 5, 5, 5), 0, 0));        
        panelActions.add(buttonClose,     Layout.getGBC( 4, 0, 1, 1, 1.0, 1.0
                        ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
						 new Insets(5, 5, 5, 5), 0, 0));
        
        
        //**********************************************************
        //Main Panel
        //**********************************************************
        if (panelSearch != null) {
            getContentPane().add(panelSearch,   Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                 new Insets(5, 5, 5, 5), 0, 0));
        }
        getContentPane().add(panelSettings, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                             new Insets(5, 5, 5, 5), 0, 0));  
        getContentPane().add(panelResults,  Layout.getGBC( 0, 2, 1, 1, 50.0, 50.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                             new Insets(5, 5, 5, 5), 0, 0));
        getContentPane().add(panelActions,  Layout.getGBC( 0, 3, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
							 new Insets(5, 5, 5, 5), 0, 0));
        getContentPane().add(panelProgress, Layout.getGBC( 0, 4, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                             new Insets(5, 5, 5, 5), 0, 0));
        getContentPane().add(panelLog,      Layout.getGBC( 0, 5, 1, 1, 10.0, 10.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                             new Insets(5, 5, 5, 5), 0, 0));        
        pack();
        

    }
    
    private class CancelAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            close();
        }
    }    
    
    private class StopSearchAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            stopSearch();
        }
    }
    
    private class ClearAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            clearTable();
        }
    }    

    private class UpdateAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            update();   
        }
    }   
    
    private class AddNewAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            addNew();   
        }
    }    

    private class TableMouseListener implements MouseListener {
        public void mouseReleased(MouseEvent e) {
            if (e.getClickCount() == 2) {
                openItem();
            }
        }
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {}
        public void mouseClicked(MouseEvent e) {}
    }    
}
