/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.processes.cdimport;

import java.io.File;
import java.util.Enumeration;
import java.util.List;
import java.util.Vector;

import net.sf.dc.console.da.ImportCDDialog;
import net.sf.dc.core.modules.DCModules;
import net.sf.dc.core.objects.DCObject;
import net.sf.dc.core.objects.musicfiles.MusicFile;
import net.sf.dc.messages.Messages;
import net.sf.ext.helliker.id3.MP3File;
import net.sf.ext.util.StringUtils;
import entagged.audioformats.AudioFile;
import entagged.audioformats.AudioFileIO;
import entagged.audioformats.Tag;
import entagged.audioformats.generic.TagTextField;

/**
 * Imports data from a CD (or harddisk) location into the insert table
 * of the owner module
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version 0.50
 */
public class ImportMusicFilesCD extends ImportCDProcess {

    /**
     * Initializes (but does  not run) the Movie CD import process
     * @param ui used for displaying the progress
     * @param sourceDir directory which is used as source for the
     *        import
     * @param confirmEachDir indicates whether for each directory
     *        confirmation should be asked if the underlying data
     *        of this directory should be read or not.
     * @param cdName cd name (used as the location name)
     */   
    public ImportMusicFilesCD( ImportCDDialog ui, 
                               String sourceDir,
            			       boolean confirmEachDir,
							   String cdName) {
    	super(ui, sourceDir, confirmEachDir, cdName);
    }

    protected int getModule() {
        return DCModules._MUSIC;
    }


    public String[] getExtensions() {
        return new String[] {"mp3", "ogg", "mp4", "flac", "flc", "ape", "asf", "wav", "mpc"};
    }
    
    /**
     * Parses the source file for information. 
     */   
    protected DCObject parseInformation(String filename) {
        String[] extensions = getExtensions();
        boolean equals = false;
        for (int i = 0; i < extensions.length; i++) {
            if (filename.toLowerCase().endsWith(extensions[i])) {
            	equals = true;
                break;
            }
        }
        
        if (!equals) {
    		return null;
    	}
    	
        MusicFile music = read(filename);
        music.setValue(MusicFile._L_LOCATION, cdName);
        music.setValue(MusicFile._F_FILENAME, filename);
        music.setValue(MusicFile._K_FILESIZE, new Integer(getFileSize(filename)));
        return music;
    }
    
    /**
     * Parses the file with two seperate libraries and merges the result
     * @param sFile file to be parse
     */
    private MusicFile read(String sFile) {
        File file = new File(sFile);
        MusicFile f1 = parseWithEntagged(file);
        
        if (!sFile.endsWith(".mp3")) {
        	return f1;
        } else {
        	MusicFile f2 = parseWithJD3(file);
            Object[] values1 = f1.getValues();
            Object[] values2 = f2.getValues();
            
            MusicFile musicFile = new MusicFile();
            for (int i = 1; i < values1.length; i++) {
            	if (   values1[i] != null &&
                       !values1[i].equals("") && 
                       !values1[i].equals(new Integer[0])) {
                    musicFile.setValue(i, values1[i]);
                } else {
                    musicFile.setValue(i, values2[i]);   
                }
            }
            if (    f2.getValue(MusicFile._E_GENRE) != null &&
            		!StringUtils.isEmpty(f2.getValue(MusicFile._E_GENRE).toString())) {
                musicFile.setValue(MusicFile._E_GENRE, f2.getValue(MusicFile._E_GENRE));
            }
            
            
            f1 = null;
            f2 = null;
            
            return musicFile;
        } 
    }
    
    /**
     * Parse with the Entagged library
     * @param file audio file
     */
    private MusicFile parseWithEntagged(File file) {
        MusicFile music = new MusicFile();
        
        try {
            AudioFile audioFile = AudioFileIO.read(file);
            
            music.setValue(MusicFile._H_QUALITY, new Integer(audioFile.getBitrate()));
            music.setValue(MusicFile._G_PLAYLENGTH, new Integer(audioFile.getLength()));
            music.setValue(MusicFile._I_ENCODING, audioFile.getEncodingType());
    
            Tag tag = audioFile.getTag();
            if (tag != null) {
                String album = getBest(tag.getAlbum());
                String artist = getBest(tag.getArtist());
                String genre = getBest(tag.getGenre());
                String year = getBest(tag.getYear());
                String title = getBest(tag.getTitle());
                
                music.setValue(MusicFile._A_TITLE, title);
                music.setValue(MusicFile._B_ARTIST, artist);
                music.setValue(MusicFile._C_ALBUM, album);
                music.setValue(MusicFile._D_YEAR, year);
                music.setValue(MusicFile._E_GENRE, genre);                    
            }
        } catch (Exception ignore) {}
        
        return music;
    }
    
    private String getBest(List list) {
    	String s = "";

        Vector v = new Vector(list);
        v.trimToSize();     
        
        for (Enumeration enum = v.elements(); enum.hasMoreElements(); ) {
            try {
                TagTextField tagField = (TagTextField) enum.nextElement();
                s = tagField.getContent();
            } catch (Exception exp) {
            	Messages.add(exp);
            }
        }
        
        return s;
    }
    
    /**
     * Parse with the JD3 library
     * @param file audio file
     */
    private MusicFile parseWithJD3(File file) {
        MusicFile music = new MusicFile();
        
        try {
            MP3File mp3File = new MP3File(file);
            music.setValue(MusicFile._H_QUALITY, new Integer(mp3File.getBitRate()));
            music.setValue(MusicFile._G_PLAYLENGTH, new Integer(new Long(mp3File.getPlayingTime()).intValue()));        
            music.setValue(MusicFile._A_TITLE, mp3File.getTitle());
            music.setValue(MusicFile._B_ARTIST, mp3File.getArtist());
            music.setValue(MusicFile._C_ALBUM, mp3File.getAlbum());
            music.setValue(MusicFile._D_YEAR, mp3File.getYear());
            music.setValue(MusicFile._E_GENRE, mp3File.getGenre());
            music.setValue(MusicFile._I_ENCODING, mp3File.getMPEGLayer() + " " + mp3File.getMPEGVersion());
        } catch (Exception ignore) {}
        
        return music;
    }
}
