/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.ext.jflexsett.settings;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.io.File;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

/**
 * Contains all Settings Groups and there settings. 
 * The Settings class is the only class which needs to be referenced
 * to add groups, add settings, get settings and get values of settings.
 * 
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class Settings {

    private static File settingsFile = new File("unnamed.ini");
    private static Hashtable htGroups = new Hashtable();
    
    /**
     * Adds a group to the hashtable. A group may contain sub-groups.
     * Only the parent, to which a sub-group belongs, should be added.
     * 
     * @param sKey unique identifier for this group
     * @param group a top-level group (parent)
     */ 
    public static void addGroup(String sKey, SettingsGroup group) {
        htGroups.put(sKey, group);
    }

    /**
     * Specifies the location and name of the settings file
     * 
     * @param file the settings file
     */
    public static void setSettingsFile(File file) {
        settingsFile = file;
    }
    
    public static File getSettingsFile() {
        return settingsFile;   
    }

    /**
     * Retrieves a settings group with the key
     */
    private static SettingsGroup getSettingGroup(String sGroupKey) {
        Vector vSettings = getFlatSettingsGroups();
        for (Enumeration enum = vSettings.elements(); enum.hasMoreElements(); ) {
            SettingsGroup settingsGroup = (SettingsGroup) enum.nextElement();
            if (settingsGroup.getKey().equals(sGroupKey)) {
                return settingsGroup; 
            }
        }
        return null;
    }

    /**
     * Adds a setting to a specified group. A group can be either
     * a parent or a child.
     * 
     * @param sGroupKey the key of the group
     * @param setting the setting to be added to the specified group
     */
    public static void addSetting(String sGroupKey, Setting setting) {
        SettingsGroup stGroup = getSettingGroup(sGroupKey);
        stGroup.add(setting);
    }

    public static boolean isSettingKeyValid(String sKey) {
        boolean bValid = getSetting(sKey) == null ? false : true;
        return bValid;
    }

    /**
     * Retrieves a setting
     * 
     * @param sSettingsKey the settings key
     */
    public static Setting getSetting(String sSettingsKey) {
        for (Enumeration enum = htGroups.elements(); enum.hasMoreElements(); ) {
            SettingsGroup group = (SettingsGroup) enum.nextElement();
            Setting setting = group.getSetting(sSettingsKey);
            
            if (setting != null) return setting;
        }
        return null;
    }

    /**
     * Retrieves a value of a setting
     * 
     * @param sKey the settings key
     */
    public static Object getValue(String sKey) {
        Setting setting = getSetting(sKey);
        Object oValue = setting != null ? setting.getValue() : null;
        return oValue;
    }

    /**
     * Returns the value of the setting as a String array
     * 
     * @param sKey the settings key
     */
    public static String[] getValueAsStringArray(String sKey) {
        Object obj = getValue(sKey);
        String[] sArray = (String[]) obj;
        return sArray;
    }
    
    public static int[] getValueAsIntegerArray(String sKey) {
        Object obj = getValue(sKey);
        int[] iArray = (int[]) obj;
        return iArray;
    }

    public static Font getValueAsFont(String sKey) {
        Object obj = getValue(sKey);
        return (Font) obj;
    }    
    
    /**
     * Returns the value of the setting as an integer
     * 
     * @param sKey the settings key
     */
    public static int getValueAsInt(String sKey) {
        Object obj = getValue(sKey);
        if (obj == null) {
        	return -1;   
        } else {
        	int iValue = ((Integer) obj).intValue();
        	return iValue;
        }
    }

    /**
     * Returns the value of the setting as a boolean
     * 
     * @param sKey the settings key
     */
    public static boolean getValueAsBoolean(String sKey) {
        Object obj = getValue(sKey);
        boolean bValue = ((Boolean) obj).booleanValue();
        return bValue;
    }

    /**
     * Returns the value of the setting as a color object
     * 
     * @param sKey the settings key
     */
    public static Color getValueAsColor(String sKey) {
        Color  color = (Color) getValue(sKey);
        return color;
    }
    
    /**
     * Returns the value of the setting as a String
     * 
     * @param sKey the settings key
     */
    public static Dimension getValueAsDimension(String sKey) {
        Dimension dim = (Dimension) getValue(sKey);
        return dim;
    }

    /**
     * Returns the value of the setting as a String
     * 
     * @param sKey the settings key
     */
    public static String getValueAsString(String sKey) {
        Object obj = getValue(sKey);
        if (obj == null) {
        	return "";   
        } else {
        	return obj.toString();
        }
    }

    /**
     * Sets the value of a setting
     * 
     * @param sKey the settings key
     * @param oValue the value 
     */
    public static void setValue(String sKey, Object oValue) {
        Setting setting = getSetting(sKey);
        if (setting != null) {
            setting.setValue(oValue);
        }
    }

    /**
     * Sets a string as a value for the setting (by parsing the 
     * string)
     * 
     * @param sKey the settings key
     * @param sValue the string value to be parsed
     */
    public static void setStringAsValue(String sKey, String sValue) {
        Setting setting = getSetting(sKey);
        if (setting != null) {
            setting.setStringAsValue(sValue);
        }
    }

    /**
     * Retrieves all the top-level groups
     */    
    public static Hashtable getSettingsGroups() {
        return htGroups;   
    }

    /**
     * Retrieves all settings groups without an hierarchy
     */
    public static Vector getFlatSettingsGroups() {
        Vector vSettings = new Vector();
        vSettings.addAll(htGroups.values());
        for (Enumeration enumGroups = htGroups.elements(); enumGroups.hasMoreElements(); ) {
            SettingsGroup settingsGroup = (SettingsGroup) enumGroups.nextElement();
            vSettings.addAll(settingsGroup.getChildren().values());
        }
        return vSettings;
    }
    
    /**
     * Retrieves all the settings
     */
    public static Vector getSettings() {
        Vector vGroups = getFlatSettingsGroups();
        Vector vSettings = new Vector(0);
        for (Enumeration enum = vGroups.elements(); enum.hasMoreElements(); ) {
            SettingsGroup settingsGroup = (SettingsGroup) enum.nextElement();
            vSettings.addAll(settingsGroup.getSettings().values());
        }        
        return vSettings;
    }
}
